/*
 * Name: Mike Cifelli
 * Course: CIS 443 - Programming Languages
 * Assignment: Lisp Interpreter 1
 */

package eval;

import parser.*;
import sexpression.Cons;
import sexpression.SExpression;
import sexpression.Symbol;

/**
 * <code>CDR</code> represents the CDR function in Lisp.
 */
public class CDR extends LispFunction {

    // The number of arguments that CDR takes.
    private static final int NUM_ARGS = 1;

    public SExpression call(Cons argList) {
        // retrieve the number of arguments passed to CDR
        int argListLength = LENGTH.getLength(argList);

        // make sure we have received the proper number of arguments
        if (argListLength != NUM_ARGS) {
            Cons originalSExpr = new Cons(new Symbol("CDR"), argList);
            String errMsg = "too " +
                            ((argListLength > NUM_ARGS) ? "many" : "few") +
                            " arguments given to CDR: " + originalSExpr;

            throw new RuntimeException(errMsg);
        }

        SExpression argCar = argList.getCar();

        // make sure that the argument is a list
        if (argCar.listp()) {
            Cons arg = (Cons) argCar;

            return arg.getCdr();
        }

        // the argument is not a list
        throw new RuntimeException("CDR: " + argCar + " is not a list");
    }

}
