package parser;

import java.text.MessageFormat;

import error.LispException;
import token.Token;

public abstract class MalformedSExpressionException extends LispException {

    private static final long serialVersionUID = 1L;
    private Token token;

    public MalformedSExpressionException(Token token) {
        this.token = token;
    }

    @Override
    public int getSeverity() {
        return 0;
    }

    @Override
    public String getMessage() {
        return MessageFormat.format("{0} - line {1}, column {2}", getMessagePrefix(), token.getLine(),
                                    token.getColumn());
    }

    public abstract String getMessagePrefix();

    public static class EofEncounteredException extends MalformedSExpressionException {

        private static final long serialVersionUID = 1L;

        public EofEncounteredException(Token token) {
            super(token);
        }

        public String getMessagePrefix() {
            return "end-of-file encountered";
        }
    }

    public static class StartsWithRightParenthesisException extends MalformedSExpressionException {

        private static final long serialVersionUID = 1L;

        public StartsWithRightParenthesisException(Token token) {
            super(token);
        }

        public String getMessagePrefix() {
            return "Expression begins with ')'";
        }
    }

    public static class UnrecognizedTokenException extends MalformedSExpressionException {

        private static final long serialVersionUID = 1L;

        public UnrecognizedTokenException(Token token) {
            super(token);
        }

        public String getMessagePrefix() {
            return "Unrecognized token";
        }
    }

}